<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Model;

use Magento\Framework\Model\AbstractModel;
use FiloBlu\MappConnector\Helper\ExceptionHelper;
use FiloBlu\MappConnector\Helper\LogHelper;

/**
 * Class Jobs
 * @package FiloBlu\MappConnector\Model
 */
class Jobs extends AbstractModel{


    /**
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource $resource
     * @param \Magento\Framework\Data\Collection\Db $resourceCollection
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }

    /**
     *
     */
    public function _construct()
    {
        $this->_init(\FiloBlu\MappConnector\Model\ResourceModel\Jobs::class);
    }

    /**
     * @param $job
     * @return $this
     */
    public function insertJob($job){
        return $this->addData($job)->save();
    }

    /**
     * Updates the job status
     *
     * @param $id
     * @param string $request
     * @param string $response
     * @param string $log
     * @param string $status
     */
    public function updateJob($id,$request='',$response='',$log=[],$status='processed') {
        $resp = ($response==null)?null:(is_array($response))?json_encode($response):json_encode($response->getData()['response']->getBody());

        if($resp != null) {
            if(is_array($response)){
                $r = json_decode($response[3]);
            }
            else {
                $r = json_decode(json_decode($resp));
            }

            if(isset($r->errorCode) && !empty($r->errorCode)){
                $status = 'error';
            }
        }

        $req = null;
        if(!empty($request)){
            $req = json_encode($request);
        }

        $this->load($id)
            ->setStatus($status)
            ->setResponse($resp)
            ->setRequest($req)
            ->setLog((count($log))?implode('<br/>',$log):null)
            ->save();
    }

    /**
     * @param        $ids
     * @param string $status
     * @param string $mode
     */
    public function massUpdateJob($ids ,$status='waiting', $mode = 'ftp') {
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance(); // Instance of object manager
        $resource = $objectManager->get('Magento\Framework\App\ResourceConnection');
        $connection = $resource->getConnection();
        $tableName = $resource->getTableName('filoblu_mappconnector_jobs'); //gives table name with prefix

        $id_list = '';
        if(count($ids)>0){
            foreach ($ids as $id){
                $id_list .= "'".$id."',";
            }
            $id_list = rtrim($id_list,',');
        }

        if(!empty($id_list)){
            $sql = "UPDATE $tableName SET mode = '$mode', status = '$status' WHERE id IN ($id_list) ";
            $connection->query($sql);
        }
    }

    public function getAllCompletedRestIds(){
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance(); // Instance of object manager
        $resource = $objectManager->get('Magento\Framework\App\ResourceConnection');
        $connection = $resource->getConnection();
        $tableName = $resource->getTableName('filoblu_mappconnector_jobs'); //gives table name with prefix

        $sql = "SELECT id FROM $tableName WHERE mode = 'rest' AND (status = 'processed' OR status = 'error') ";
        $ids = $connection->fetchCol($sql);

        return $ids;
    }

    /**
     * Update Job and reset log
     *
     * @param $response
     */
    public function updateJobAndResetLog($job,$request,$response){
        $status = (ExceptionHelper::getErrorStatus())?'error':(($response == null)?'skipped':'processed');
        $log = [];
        if(count(LogHelper::getLogBuffer())) {
            $log = array_merge(['LOGS:', '----------------------------', ''], LogHelper::getLogBuffer(),['']);
        }
        if(count(ExceptionHelper::getErrorBuffer())){
            $log = array_merge($log, ['ERRORS:', '----------------------------', ''], ExceptionHelper::getErrorBuffer());
        }

        $this->updateJob(
            $job['id'],
            $request,
            $response,
            $log,
            $status);
        ExceptionHelper::resetErrorBuffer();
        ExceptionHelper::resetErrorStatus();
        LogHelper::resetLogBuffer();
        return "Job ".$job['id']." processed. status -> ".strtoupper($status);
    }

    /**
     * @param $config
     * @return string
     */
    public function cleanRecords($config){
        if($config->getConfigValue('mappconnector/general/enable_clean')){
            $job_status = $config->getConfigValue('mappconnector/general/job_status_to_clean');
            $time_range = $config->getConfigValue('mappconnector/general/clean_time_range');
            if(!empty($job_status)){

                $objectManager = \Magento\Framework\App\ObjectManager::getInstance(); // Instance of object manager
                $resource = $objectManager->get('Magento\Framework\App\ResourceConnection');
                $connection = $resource->getConnection();
                $tableName = $resource->getTableName('filoblu_mappconnector_jobs'); //gives table name with prefix

                $j = explode(',',$job_status);
                $in = [];
                foreach($j as $k => $v){
                    $in[$k] = "'".$v."'";
                }

                if($time_range != 'all'){
                    $date = "AND updated_at <= NOW() - INTERVAL ".$time_range." DAY";
                }

                $sql_select = 'SELECT count(*) as count FROM '.$tableName.' WHERE status IN ('.implode(',',$in).') '.$date;
                $result = $connection->fetchAll($sql_select);

                $sql = 'DELETE FROM '.$tableName.' WHERE status IN ('.implode(',',$in).') '.$date;
                $connection->query($sql);

                return $result[0]['count'].' jobs deleted';

            }
            else {
                return 'Empty job status in config';
            }
        }
        else {
            return 'Clean disabled by config';
        }
    }

    /**
     * @param $entity
     * @param $id
     *
     * @return mixed
     */
    public function getJobByEntityId($entity,$id){
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance(); // Instance of object manager
        $resource = $objectManager->get('Magento\Framework\App\ResourceConnection');
        $connection = $resource->getConnection();
        $tableName = $resource->getTableName('filoblu_mappconnector_jobs'); //gives table name with prefix

        $sql = "SELECT id FROM $tableName  WHERE entity = '$entity'  AND entity_id = $id";

        $result = $connection->fetchAll($sql);

        return $result;
    }

    /**
     * Prepare all record for full export
     *
     * @param $entity
     * @param $records
     * @param $key
     * @return int
     */
    public function prepareExport($entity,$records,$key,$type,$mode = 'ftp'){
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance(); // Instance of object manager
        $resource = $objectManager->get('Magento\Framework\App\ResourceConnection');
        $connection = $resource->getConnection();
        $tableName = $resource->getTableName('filoblu_mappconnector_jobs'); //gives table name with prefix

        //Full incremental let's a full export in a more fast way
        if($type == 'fullincremental') {
            $type = 'full';
            $sql = "UPDATE $tableName SET mode = '$mode', status = 'waiting', type = '$type', request = null , response = null , log = null  WHERE entity = '$entity' ";
            $connection->query($sql);
        }

        $i = 0;
        foreach($records as $record){
            $sql = "INSERT INTO $tableName (entity,entity_id,mode,status,type) VALUES ('$entity', '".$record[$key]."', '".$mode."', 'waiting','".$type."') ON DUPLICATE KEY UPDATE status = 'waiting', type = '$type' , mode = '$mode' , request = null , response = null , log = null ";
            $connection->query($sql);
            $i++;
        }

        return $i;
    }

}